@@ -58,6 +58,7 @@ class LocalCluster(object):
     Shut down the extra worker
     >>> c.remove_worker(w)  # doctest: +SKIP
     """
+
     def __init__(self, n_workers=None, threads_per_worker=None, processes=True,
                  loop=None, start=True, ip=None, scheduler_port=0,
                  silence_logs=logging.CRITICAL, diagnostics_port=8787,
@@ -85,7 +86,8 @@ class LocalCluster(object):
 
         self.loop = loop or IOLoop()
         if start and not self.loop._running:
-            self._thread = Thread(target=self.loop.start)
+            self._thread = Thread(target=self.loop.start,
+                                  name="LocalCluster loop")
             self._thread.daemon = True
             self._thread.start()
             while not self.loop._running:
@@ -173,9 +175,13 @@ class LocalCluster(object):
 
         self.workers.append(w)
 
-        while w.worker_address not in self.scheduler.worker_info:
+        while w.status != 'closed' and w.worker_address not in self.scheduler.worker_info:
             yield gen.sleep(0.01)
 
+        if w.status == 'closed':
+            self.workers.remove(w)
+            raise gen.TimeoutError("Worker failed to start")
+
         raise gen.Return(w)
 
     def start_worker(self, ncores=0, **kwargs):
@@ -202,7 +208,8 @@ class LocalCluster(object):
     @gen.coroutine
     def _stop_worker(self, w):
         yield w._close()
-        self.workers.remove(w)
+        if w in self.workers:
+            self.workers.remove(w)
 
     def stop_worker(self, w):
         """ Stop a running worker
@@ -242,9 +249,15 @@ class LocalCluster(object):
         if self.loop._running:
             sync(self.loop, self._close)
         if hasattr(self, '_thread'):
-            sync(self.loop, self.loop.stop)
-            self._thread.join(timeout=1)
-            self.loop.close()
+            if self.loop._running:
+                self.loop.add_callback(self.loop.stop)
+            try:
+                self._thread.join(timeout=1)
+            finally:
+                try:
+                    self.loop.close()
+                except ValueError:
+                    pass
             del self._thread
 
     @gen.coroutine
@@ -271,8 +284,8 @@ class LocalCluster(object):
         """
         workers = set(workers)
         yield [self._stop_worker(w)
-                for w in self.workers
-                if w.worker_address in workers]
+               for w in self.workers
+               if w.worker_address in workers]
         while workers & set(self.workers):
             yield gen.sleep(0.01)
 
