@@ -7,10 +7,26 @@ from tornado.ioloop import IOLoop
 
 from distributed import Client
 from distributed.deploy import Adaptive, LocalCluster
-from distributed.utils_test import loop, slowinc, gen_test
+from distributed.utils_test import gen_test, slowinc
+from distributed.utils_test import loop  # flake8: noqa
 from distributed.metrics import time
 
 
+def test_get_scale_up_kwargs(loop):
+    with LocalCluster(0, scheduler_port=0, silence_logs=False,
+                      diagnostics_port=None, loop=loop) as cluster:
+
+        alc = Adaptive(cluster.scheduler, cluster, interval=100,
+                       scale_factor=3)
+        assert alc.get_scale_up_kwargs() == {'n': 1}
+
+        with Client(cluster, loop=loop) as c:
+            future = c.submit(lambda x: x + 1, 1)
+            assert future.result() == 2
+            assert c.ncores()
+            assert alc.get_scale_up_kwargs() == {'n': 3}
+
+
 def test_adaptive_local_cluster(loop):
     with LocalCluster(0, scheduler_port=0, silence_logs=False,
                       diagnostics_port=None, loop=loop) as cluster:
@@ -41,8 +57,7 @@ def test_adaptive_local_cluster_multi_workers():
                            diagnostics_port=None, loop=loop, start=False)
     cluster.scheduler.allowed_failures = 1000
     alc = Adaptive(cluster.scheduler, cluster, interval=100)
-    c = Client(cluster, start=False, loop=loop)
-    yield c._start()
+    c = yield Client(cluster, asynchronous=True, loop=loop)
 
     futures = c.map(slowinc, range(100), delay=0.01)
 
@@ -68,5 +83,5 @@ def test_adaptive_local_cluster_multi_workers():
     futures = c.map(slowinc, range(100), delay=0.01)
     yield c._gather(futures)
 
-    yield c._shutdown()
+    yield c._close()
     yield cluster._close()
