@@ -2,19 +2,17 @@ from __future__ import print_function, division, absolute_import
 
 from time import sleep
 
-import pytest
 from tornado import gen
 
 from distributed import Client, Scheduler, Worker
 from distributed.diagnostics.progressbar import TextProgressBar, progress
 from distributed.metrics import time
-from distributed.utils_test import (cluster, loop, inc,
-        div, dec, gen_cluster)
-from distributed.worker import dumps_task
+from distributed.utils_test import (cluster, inc, div, gen_cluster)
+from distributed.utils_test import loop  # flake8: noqa
 
 
 def test_text_progressbar(capsys, loop):
-    with cluster(nanny=True) as (s, [a, b]):
+    with cluster(nanny=True, should_check_state=False) as (s, [a, b]):
         with Client(s['address'], loop=loop) as c:
             futures = c.map(inc, range(10))
             p = TextProgressBar(futures, interval=0.01, complete=True)
@@ -76,12 +74,12 @@ def test_TextProgressBar_empty(loop, capsys):
 def check_bar_completed(capsys, width=40):
     out, err = capsys.readouterr()
     bar, percent, time = [i.strip() for i in out.split('\r')[-1].split('|')]
-    assert bar == '[' + '#'*width + ']'
+    assert bar == '[' + '#' * width + ']'
     assert percent == '100% Completed'
 
 
 def test_progress_function(loop, capsys):
-    with cluster() as (s, [a, b]):
+    with cluster(should_check_state=False) as (s, [a, b]):
         with Client(s['address'], loop=loop) as c:
             f = c.submit(lambda: 1)
             g = c.submit(lambda: 2)
