@@ -1,19 +1,19 @@
 """ This file is experimental and may disappear without warning """
 from __future__ import print_function, division, absolute_import
 
-import logging
-
 from dask.base import tokenize
 from dask.bytes import core
-from dask.utils import infer_storage_options
 from hdfs3 import HDFileSystem
 
+# infer_storage_options moved after dask 0.14.3 release
+try:
+    from dask.bytes.utils import infer_storage_options
+except ImportError:
+    from dask.utils import infer_storage_options
+
 from .utils import PY3
 
 
-logger = logging.getLogger(__name__)
-
-
 class DaskHDFileSystem(HDFileSystem):
 
     sep = '/'
@@ -40,7 +40,7 @@ class DaskHDFileSystem(HDFileSystem):
             part.append(parts)
             try:
                 self.mkdir('/'.join(part))
-            except:
+            except Exception:
                 pass
 
     def glob(self, path):
