@@ -22,15 +22,16 @@ which is included as a comment at the end of this file:
 """
 from __future__ import print_function, division, absolute_import
 
-from concurrent.futures import thread
+from . import _concurrent_futures_thread as thread
 import logging
-from threading import local, Thread
+import threading
 
 from .compatibility import get_thread_identity
+from .metrics import time
 
 logger = logging.getLogger(__name__)
 
-thread_state = local()
+thread_state = threading.local()
 
 
 def _worker(executor, work_queue):
@@ -57,24 +58,42 @@ def _worker(executor, work_queue):
 class ThreadPoolExecutor(thread.ThreadPoolExecutor):
     def _adjust_thread_count(self):
         if len(self._threads) < self._max_workers:
-            t = Thread(target=_worker,
-                       name="ThreadPool worker %d" % len(self._threads,),
-                       args=(self, self._work_queue))
+            t = threading.Thread(target=_worker,
+                                 name="ThreadPool worker %d" % len(self._threads,),
+                                 args=(self, self._work_queue))
             t.daemon = True
             self._threads.add(t)
             t.start()
 
+    def shutdown(self, wait=True, timeout=None):
+        with threads_lock:
+            with self._shutdown_lock:
+                self._shutdown = True
+                self._work_queue.put(None)
+            if timeout is not None:
+                deadline = time() + timeout
+            for t in self._threads:
+                if timeout is not None:
+                    timeout2 = max(deadline - time(), 0)
+                else:
+                    timeout2 = None
+                t.join(timeout=timeout2)
+
 
 def secede():
     """ Have this thread secede from the ThreadPoolExecutor """
     thread_state.proceed = False
     ident = get_thread_identity()
-    for t in list(thread_state.executor._threads):
-        if t.ident == ident:
-            thread_state.executor._threads.remove(t)
-            break
+    with threads_lock:
+        for t in list(thread_state.executor._threads):
+            if t.ident == ident:
+                thread_state.executor._threads.remove(t)
+                break
+        thread_state.executor._adjust_thread_count()
 
 
+threads_lock = threading.Lock()
+
 """
 PSF LICENSE AGREEMENT FOR PYTHON 3.5.2
 ======================================
