@@ -1,5 +1,6 @@
 from __future__ import print_function, division, absolute_import
 
+import bisect
 from collections import defaultdict, deque
 from datetime import timedelta
 import heapq
@@ -26,10 +27,12 @@ from tornado.locks import Event
 
 from .batched import BatchedSend
 from .comm import get_address_host, get_local_address_for
+from .comm.utils import offload
 from .config import config
 from .compatibility import unicode, get_thread_identity
 from .core import (error_message, CommClosedError,
                    rpc, pingpong, coerce_to_address)
+from .diagnostics import profile
 from .metrics import time
 from .node import ServerNode
 from .preloading import preload_modules
@@ -40,7 +43,8 @@ from .sizeof import safe_sizeof as sizeof
 from .threadpoolexecutor import ThreadPoolExecutor, secede as tpe_secede
 from .utils import (funcname, get_ip, has_arg, _maybe_complex, log_errors,
                     ignoring, validate_key, mp_context, import_file,
-                    silence_logging, thread_state, json_load_robust)
+                    silence_logging, thread_state, json_load_robust, key_split,
+                    format_bytes)
 from .utils_comm import pack_data, gather_from_workers
 
 _ncores = mp_context.cpu_count()
@@ -76,7 +80,7 @@ class WorkerBase(ServerNode):
                  heartbeat_interval=5000, reconnect=True, memory_limit='auto',
                  executor=None, resources=None, silence_logs=None,
                  death_timeout=None, preload=(), security=None,
-                 contact_address=None, **kwargs):
+                 contact_address=None, memory_monitor_interval=200, **kwargs):
         if scheduler_file:
             cfg = json_load_robust(scheduler_file)
             scheduler_addr = cfg['address']
@@ -91,6 +95,7 @@ class WorkerBase(ServerNode):
         self.death_timeout = death_timeout
         self.preload = preload
         self.contact_address = contact_address
+        self.memory_monitor_interval = memory_monitor_interval
         if silence_logs:
             silence_logging(level=silence_logs)
 
@@ -106,12 +111,13 @@ class WorkerBase(ServerNode):
         self.listen_args = self.security.get_listen_args('worker')
 
         if memory_limit == 'auto':
-            memory_limit = int(TOTAL_MEMORY * 0.6 * min(1, self.ncores / _ncores))
+            memory_limit = int(TOTAL_MEMORY * min(1, self.ncores / _ncores))
         with ignoring(TypeError):
             memory_limit = float(memory_limit)
         if isinstance(memory_limit, float) and memory_limit <= 1:
             memory_limit = memory_limit * TOTAL_MEMORY
         self.memory_limit = memory_limit
+        self.paused = False
 
         if self.memory_limit:
             try:
@@ -120,7 +126,7 @@ class WorkerBase(ServerNode):
                 raise ImportError("Please `pip install zict` for spill-to-disk workers")
             path = os.path.join(self.local_dir, 'storage')
             storage = Func(serialize_bytelist, deserialize_bytes, File(path))
-            self.data = Buffer({}, storage, int(float(self.memory_limit)), weight)
+            self.data = Buffer({}, storage, int(float(self.memory_limit) * 0.6), weight)
         else:
             self.data = dict()
         self.loop = loop or IOLoop.current()
@@ -130,6 +136,8 @@ class WorkerBase(ServerNode):
         self.executor = executor or ThreadPoolExecutor(self.ncores)
         self.scheduler = rpc(scheduler_addr, connection_args=self.connection_args)
         self.name = name
+        self.pause_fraction = config.get('worker-pause-memory-fraction', 0.8)
+        self.scheduler_delay = 0
         self.heartbeat_interval = heartbeat_interval
         self.heartbeat_active = False
         self.execution_state = {'scheduler': self.scheduler.address,
@@ -159,6 +167,9 @@ class WorkerBase(ServerNode):
             'health': self.host_health,
             'upload_file': self.upload_file,
             'start_ipython': self.start_ipython,
+            'call_stack': self.get_call_stack,
+            'profile': self.get_profile,
+            'profile_metadata': self.get_profile_metadata,
             'keys': self.keys,
         }
 
@@ -172,6 +183,12 @@ class WorkerBase(ServerNode):
         self.periodic_callbacks['heartbeat'] = pc
         self._address = contact_address
 
+        self._memory_monitoring = False
+        pc = PeriodicCallback(self.memory_monitor,
+                              self.memory_monitor_interval,
+                              io_loop=self.loop)
+        self.periodic_callbacks['memory'] = pc
+
     @property
     def worker_address(self):
         """ For API compatibility with Nanny """
@@ -191,7 +208,8 @@ class WorkerBase(ServerNode):
                 else:
                     kwargs = {}
 
-                yield self.scheduler.register(
+                start = time()
+                response = yield self.scheduler.register(
                     address=self.contact_address,
                     name=self.name,
                     ncores=self.ncores,
@@ -204,6 +222,9 @@ class WorkerBase(ServerNode):
                     ready=len(self.ready),
                     in_flight=len(self.in_flight_tasks),
                     **kwargs)
+                end = time()
+                middle = (start + end) / 2
+                self.scheduler_delay = response['time'] - middle
             finally:
                 self.heartbeat_active = False
         else:
@@ -222,6 +243,7 @@ class WorkerBase(ServerNode):
             if self.status in ('closed', 'closing'):
                 raise gen.Return
             try:
+                _start = time()
                 future = self.scheduler.register(
                         ncores=self.ncores,
                         address=self.contact_address,
@@ -239,7 +261,10 @@ class WorkerBase(ServerNode):
                     diff = self.death_timeout - (time() - start)
                     future = gen.with_timeout(timedelta(seconds=diff), future,
                                               io_loop=self.loop)
-                resp = yield future
+                response = yield future
+                _end = time()
+                middle = (_start + _end) / 2
+                self.scheduler_delay = response['time'] - middle
                 self.status = 'running'
                 break
             except EnvironmentError:
@@ -248,8 +273,9 @@ class WorkerBase(ServerNode):
                 yield gen.sleep(0.1)
             except gen.TimeoutError:
                 pass
-        if resp != 'OK':
-            raise ValueError("Unexpected response from register: %r" % (resp,))
+        if response['status'] != 'OK':
+            raise ValueError("Unexpected response from register: %r" %
+                             (response,))
         self.periodic_callbacks['heartbeat'].start()
 
     def start_services(self, listen_ip=''):
@@ -316,7 +342,7 @@ class WorkerBase(ServerNode):
         logger.info('-' * 49)
         logger.info('              Threads: %26d', self.ncores)
         if self.memory_limit:
-            logger.info('               Memory: %23.2f GB', self.memory_limit / 1e9)
+            logger.info('               Memory: %26s', format_bytes(self.memory_limit))
         logger.info('      Local Directory: %26s', self.local_dir)
         logger.info('-' * 49)
 
@@ -326,6 +352,9 @@ class WorkerBase(ServerNode):
             logger.info('        Registered to: %26s', self.scheduler.address)
             logger.info('-' * 49)
 
+        for pc in self.periodic_callbacks.values():
+            pc.start()
+
     def start(self, port=0):
         self.loop.add_callback(self._start, port)
 
@@ -491,8 +520,8 @@ class WorkerBase(ServerNode):
         self.outgoing_count += 1
         duration = (stop - start) or 0.5  # windows
         self.outgoing_transfer_log.append({
-            'start': start,
-            'stop': stop,
+            'start': start + self.scheduler_delay,
+            'stop': stop + self.scheduler_delay,
             'middle': (start + stop) / 2,
             'duration': duration,
             'who': who,
@@ -530,21 +559,32 @@ class WorkerBase(ServerNode):
             )
         return self._ipython_kernel.get_connection_info()
 
+    @gen.coroutine
     def upload_file(self, comm, filename=None, data=None, load=True):
         out_filename = os.path.join(self.local_dir, filename)
-        if isinstance(data, unicode):
-            data = data.encode()
-        with open(out_filename, 'wb') as f:
-            f.write(data)
-            f.flush()
+
+        def func(data):
+            if isinstance(data, unicode):
+                data = data.encode()
+            with open(out_filename, 'wb') as f:
+                f.write(data)
+                f.flush()
+            return data
+
+        if len(data) < 10000:
+            data = func(data)
+        else:
+            data = yield offload(func, data)
 
         if load:
             try:
                 import_file(out_filename)
             except Exception as e:
                 logger.exception(e)
-                return {'status': 'error', 'exception': pickle.dumps(e)}
-        return {'status': 'OK', 'nbytes': len(data)}
+                raise gen.Return({'status': 'error',
+                                  'exception': pickle.dumps(e)})
+
+        raise gen.Return({'status': 'OK', 'nbytes': len(data)})
 
     def host_health(self, comm=None):
         """ Information about worker """
@@ -688,13 +728,17 @@ def dumps_task(task):
     return to_serialize(task)
 
 
-def apply_function(function, args, kwargs, execution_state, key):
+def apply_function(function, args, kwargs, execution_state, key,
+                   active_threads, active_threads_lock, time_delay):
     """ Run a function, collect information
 
     Returns
     -------
     msg: dictionary with status, result/error, timings, etc..
     """
+    ident = get_thread_identity()
+    with active_threads_lock:
+        active_threads[ident] = key
     thread_state.start_time = time()
     thread_state.execution_state = execution_state
     thread_state.key = key
@@ -712,9 +756,11 @@ def apply_function(function, args, kwargs, execution_state, key):
                'type': type(result) if result is not None else None}
     finally:
         end = time()
-    msg['start'] = start
-    msg['stop'] = end
-    msg['thread'] = get_thread_identity()
+    msg['start'] = start + time_delay
+    msg['stop'] = end + time_delay
+    msg['thread'] = ident
+    with active_threads_lock:
+        del active_threads[ident]
     return msg
 
 
@@ -928,6 +974,8 @@ class Worker(WorkerBase):
         The type of a particular piece of data
     * **threads**: ``{key: int}``
         The ID of the thread on which the task ran
+    * **active_threads**: ``{int: key}``
+        The keys currently running on active threads
     * **exceptions**: ``{key: exception}``
         The exception caused by running a task if it erred
     * **tracebacks**: ``{key: traceback}``
@@ -955,7 +1003,7 @@ class Worker(WorkerBase):
     heartbeat_interval: int
         Milliseconds between heartbeats to scheduler
     memory_limit: int
-        Number of bytes of data to keep in memory before using disk
+        Number of bytes of memory that this worker should use
     executor: concurrent.futures.Executor
     resources: dict
         Resources that thiw worker has like ``{'GPU': 2}``
@@ -1007,6 +1055,13 @@ class Worker(WorkerBase):
         self.exceptions = dict()
         self.tracebacks = dict()
 
+        self.active_threads_lock = threading.Lock()
+        self.active_threads = dict()
+        self.profile_keys = defaultdict(profile.create)
+        self.profile_keys_history = deque(maxlen=3600)
+        self.profile_recent = profile.create()
+        self.profile_history = deque(maxlen=3600)
+
         self.priorities = dict()
         self.priority_counter = 0
         self.durations = dict()
@@ -1053,8 +1108,22 @@ class Worker(WorkerBase):
         self.outgoing_count = 0
         self._client = None
 
+        profile_cycle_interval = kwargs.pop('profile_cycle_interval',
+                                        config.get('profile-cycle-interval', 1000))
+
         WorkerBase.__init__(self, *args, **kwargs)
 
+        pc = PeriodicCallback(self.trigger_profile,
+                              config.get('profile-interval', 10),
+                              io_loop=self.loop)
+        self.periodic_callbacks['profile'] = pc
+
+        pc = PeriodicCallback(self.cycle_profile,
+                              profile_cycle_interval,
+                              io_loop=self.loop)
+        pc.start()
+        self.periodic_callbacks['profile-cycle'] = pc
+
         _global_workers.append(weakref.ref(self))
 
     def __str__(self):
@@ -1665,10 +1734,11 @@ class Worker(WorkerBase):
 
                 self.log.append(('request-dep', dep, worker, deps))
                 logger.debug("Request %d keys", len(deps))
-                start = time()
+
+                start = time() + self.scheduler_delay
                 response = yield self.rpc(worker).get_data(keys=deps,
                                                            who=self.address)
-                stop = time()
+                stop = time() + self.scheduler_delay
 
                 if cause:
                     self.startstops[cause].append(('transfer', start, stop))
@@ -1960,6 +2030,8 @@ class Worker(WorkerBase):
         return True
 
     def ensure_computing(self):
+        if self.paused:
+            return
         try:
             while self.constrained and len(self.executing) < self.ncores:
                 key = self.constrained[0]
@@ -2008,7 +2080,10 @@ class Worker(WorkerBase):
             try:
                 result = yield self.executor_submit(key, apply_function, function,
                                                     args2, kwargs2,
-                                                    self.execution_state, key)
+                                                    self.execution_state, key,
+                                                    self.active_threads,
+                                                    self.active_threads_lock,
+                                                    self.scheduler_delay)
             except RuntimeError as e:
                 executor_error = e
                 raise
@@ -2069,6 +2144,160 @@ class Worker(WorkerBase):
     # Administrative #
     ##################
 
+    @gen.coroutine
+    def memory_monitor(self):
+        """ Track this process's memory usage and act accordingly
+
+        If we rise above 70% memory use, start dumping data to disk.
+
+        If we rise above 80% memory use, stop execution of new tasks
+        """
+        if self._memory_monitoring:
+            return
+        self._memory_monitoring = True
+        total = 0
+        proc = psutil.Process()
+        frac = proc.memory_info().rss / self.memory_limit
+
+        if frac > self.pause_fraction:
+            if not self.paused:
+                logger.warn("Worker is at %d percent memory usage.  "
+                            "Stopping work. "
+                            "Process memory: %s -- Worker memory limit: %s",
+                            int(frac * 100),
+                            format_bytes(proc.memory_info().rss),
+                            format_bytes(self.memory_limit))
+                self.paused = True
+        elif self.paused:
+            logger.warn("Worker at %d percent memory usage. Restarting work. "
+                        "Process memory: %s -- Worker memory limit: %s",
+                        int(frac * 100),
+                        format_bytes(proc.memory_info().rss),
+                        format_bytes(self.memory_limit))
+            self.paused = False
+            self.ensure_computing()
+
+        if frac > 0.70:  # dump data to disk
+            target = self.memory_limit * 0.60
+            count = 0
+
+            while proc.memory_info().rss > target:
+                if not self.data.fast:
+                    logger.warn("Memory use is high but worker has no data "
+                                "to store to disk.  Perhaps some other process "
+                                "is leaking memory?  Process memory: %s -- "
+                                "Worker memory limit: %s",
+                                format_bytes(proc.memory_info().rss),
+                                format_bytes(self.memory_limit))
+                    break
+                k, v, weight = self.data.fast.evict()
+                total += weight
+                count += 1
+                yield gen.moment
+            if count:
+                logger.debug("Moved %d pieces of data data and %s to disk",
+                             count, format_bytes(total))
+
+        self._memory_monitoring = False
+        raise gen.Return(total)
+
+    def cycle_profile(self):
+        now = time() + self.scheduler_delay
+        prof, self.profile_recent = self.profile_recent, profile.create()
+        self.profile_history.append((now, prof))
+
+        self.profile_keys_history.append((now, dict(self.profile_keys)))
+        self.profile_keys.clear()
+
+    def trigger_profile(self):
+        """
+        Get a frame from all actively computing threads
+
+        Merge these frames into existing profile counts
+        """
+        if not self.active_threads:  # hope that this is thread-atomic?
+            return
+        start = time()
+        with self.active_threads_lock:
+            active_threads = self.active_threads.copy()
+        frames = sys._current_frames()
+        frames = {ident: frames[ident] for ident in active_threads}
+        for ident, frame in frames.items():
+            key = key_split(active_threads[ident])
+            profile.process(frame, None, self.profile_recent, stop='_concurrent_futures_thread.py')
+            profile.process(frame, None, self.profile_keys[key], stop='_concurrent_futures_thread.py')
+        stop = time()
+        if self.digests is not None:
+            self.digests['profile-duration'].add(stop - start)
+
+    def get_profile(self, comm=None, start=None, stop=None, key=None):
+        now = time() + self.scheduler_delay
+        if key is None:
+            history = self.profile_history
+        else:
+            history = [(t, d[key]) for t, d in self.profile_keys_history
+                       if key in d]
+        if start is None:
+            istart = 0
+        else:
+            istart = bisect.bisect_left(history, (start,))
+
+        if stop is None:
+            istop = None
+        else:
+            istop = bisect.bisect_right(history, (stop,)) + 1
+            if istop >= len(history):
+                istop = None  # include end
+
+        if istart == 0 and istop is None:
+            history = list(history)
+        else:
+            iistop = len(history) if istop is None else istop
+            history = [history[i] for i in range(istart, iistop)]
+
+        prof = profile.merge(*pluck(1, history))
+
+        if not history:
+            return profile.create()
+
+        if istop is None and (start is None or start < now):
+            if key is None:
+                recent = self.profile_recent
+            else:
+                recent = self.profile_keys[key]
+            prof = profile.merge(prof, recent)
+
+        return prof
+
+    def get_profile_metadata(self, comm=None, start=0, stop=None):
+        if stop is None:
+            add_recent = True
+        now = time() + self.scheduler_delay
+        stop = stop or now
+        start = start or 0
+        result = {'counts': [(t, d['count']) for t, d in self.profile_history
+                             if start < t < stop],
+                  'keys': [(t, {k: d['count'] for k, d in v.items()})
+                           for t, v in self.profile_keys_history
+                           if start < t < stop]}
+        if add_recent:
+            result['counts'].append((now, self.profile_recent['count']))
+            result['keys'].append((now, {k: v['count']
+                                          for k, v in self.profile_keys.items()}))
+        return result
+
+    def get_call_stack(self, comm=None, keys=None):
+        with self.active_threads_lock:
+            frames = sys._current_frames()
+            active_threads = self.active_threads.copy()
+            frames = {k: frames[ident]
+                      for ident, k in active_threads.items()}
+        if keys is not None:
+            frames = {k: frame for k, frame in frames.items() if k in keys}
+
+        result = {k: profile.call_stack(frame) for k, frame in frames.items()}
+        return result
+
     def validate_key_memory(self, key):
         assert key in self.data
         assert key in self.nbytes
